#include <raylib.h>
#include "preferred_size.h"
#include <math.h>
#include <raymath.h>
#include <rlgl.h>

int DrawTextBox(const char *text, int x, int y, int width, int height, float alignX, float alignY, Color bg)
{
  int mouseX = GetMouseX();
  int mouseY = GetMouseY();
  int isHovered = mouseX >= x && mouseX <= x + width && mouseY >= y && mouseY <= y + height;
  
  DrawRectangle(x, y, width, height, isHovered ? SKYBLUE : bg);
  DrawRectangleLinesEx((Rectangle){x, y, width, height}, 2.0f, BLACK);
  Font font = GetFontDefault();
  float fontSize = font.baseSize * 2;
  float spacing = 2;
  Vector2 textSize = MeasureTextEx(font, text, fontSize, spacing);
  float textX = x + (width - textSize.x) * alignX;
  float textY = y + (height - textSize.y) * alignY;
  Vector2 pos = {textX, textY};
  DrawTextEx(font, text, pos, fontSize, spacing, BLACK);
  if (isHovered)
  {
    return IsMouseButtonPressed(MOUSE_LEFT_BUTTON) ? 1 : -1;
  }
  return 0;
}

int main(void)
{
  int screenWidth = 600, screenHeight = 350;
  GetPreferredSize(&screenWidth, &screenHeight);
  InitWindow(screenWidth, screenHeight, "Loading models with multiple meshes");

  Camera camera = {0};
  camera.position = (Vector3){8.0f, 7.0f, 5.0f};
  camera.target = (Vector3){0.0f, 1.0f, 0.0f};
  camera.up = (Vector3){0.0f, 1.0f, 0.0f};
  camera.fovy = 45.0f;
  camera.projection = CAMERA_PERSPECTIVE;

  Model modelGLTF = LoadModel("data/quadset.glb");
  Model modelOBJ = LoadModel("data/quadset.obj");
  Texture2D texture = LoadTexture("data/palette.png");
  for (int i = 0; i < modelOBJ.materialCount; i++)
  {
    modelOBJ.materials[i].maps[MATERIAL_MAP_DIFFUSE].texture = texture;
  }

  Vector3 position = {0.0f, 0.0f, 0.0f};

  SetTargetFPS(30);

  int modelIndex = 0;
  Model model = modelGLTF;
  while (!WindowShouldClose())
  {
    if (IsPaused())
    {
      // canvas is not visible in browser - do nothing
      continue;
    }

    if (IsMouseButtonDown(MOUSE_LEFT_BUTTON))
      UpdateCamera(&camera, CAMERA_FIRST_PERSON);

    BeginDrawing();
    ClearBackground(GRAY);
    DrawRectangleGradientV(0, 0, GetScreenWidth(), GetScreenHeight(), SKYBLUE, LIGHTGRAY);

    if (DrawTextBox("GLTF", 10, 10, 100, 30, 0.5f, 0.5f, modelIndex == 0 ? YELLOW : WHITE) == 1)
    {
      model = modelGLTF;
      modelIndex = 0;
    }

    if (DrawTextBox("OBJ", 10, 40, 100, 30, 0.5f, 0.5f, modelIndex == 1 ? YELLOW : WHITE) == 1)
    {
      model = modelOBJ;
      modelIndex = 1;
    }

    int drawInfo = -1;
    for (int i = 0; i < model.meshCount; i++)
    {
      int y = 80 + i * 30;
      if (DrawTextBox(TextFormat("Mesh %d", i), 10, y, 100, 30, 0.5f, 0.5f, WHITE))
      {
        drawInfo = i;
        DrawTextBox(TextFormat("VertexCount: %d", model.meshes[i].vertexCount), 
          GetScreenWidth() - 210, GetScreenHeight() - 40, 200, 30, 0.5f, 0.5f, WHITE);
      }
    }

    BeginMode3D(camera);
    if (drawInfo < 0)
      DrawModel(model, position, 1.0f, WHITE);

    for (int i = drawInfo >= 0 ? drawInfo : 0; i < model.meshCount; i++)
    {
      BoundingBox box = GetMeshBoundingBox(model.meshes[i]);
      DrawBoundingBox(box, RED);
      
      if (drawInfo >= 0)
      {
        DrawMesh(model.meshes[i], model.materials[model.meshMaterial[i]], model.transform);
        break;
      }
    }

    EndMode3D();

    DrawTextBox("Debugging OBJ / GLTF", GetScreenWidth() - 310, 10, 300, 40, 0.5f, 0.5f, WHITE);
    EndDrawing();
  }

  UnloadModel(modelGLTF);
  UnloadModel(modelOBJ);
  CloseWindow();

  return 0;
}